@@ -37,11 +37,12 @@ namespace datetime {
 class Planner
 {
 public:
-    virtual ~Planner() =default;
+    virtual ~Planner();
     virtual core::Property<std::vector<Appointment>>& appointments() =0;
 
 protected:
-    Planner() =default;
+    Planner();
+    static void sort(std::vector<Appointment>&);
 };
 
 } // namespace datetime
