@@ -21,9 +21,15 @@
 #define INDICATOR_DATETIME_SNAP_H
 
 #include <datetime/appointment.h>
+#include <datetime/settings.h>
+
+#include <notifications/notifications.h>
+#include <notifications/sound.h>
+
+#include <gio/gio.h> // GDBusConnection
 
-#include <memory>
 #include <functional>
+#include <memory>
 
 namespace unity {
 namespace indicator {
@@ -35,13 +41,21 @@ namespace datetime {
 class Snap
 {
 public:
-    Snap();
+    Snap(const std::shared_ptr<unity::indicator::notifications::Engine>& engine,
+         const std::shared_ptr<unity::indicator::notifications::SoundBuilder>& sound_builder,
+         const std::shared_ptr<const Settings>& settings,
+         GDBusConnection* system_bus);
     virtual ~Snap();
 
-    typedef std::function<void(const Appointment&)> appointment_func;
+    enum class Response { None, Snooze, ShowApp };
+    typedef std::function<void(const Appointment&, const Alarm&, const Response&)> response_func;
     void operator()(const Appointment& appointment,
-                    appointment_func show,
-                    appointment_func dismiss);
+                    const Alarm& alarm,
+                    response_func on_response);
+
+private:
+    class Impl;
+    std::unique_ptr<Impl> impl;
 };
 
 } // namespace datetime
