@@ -27,15 +27,32 @@ namespace datetime {
 *****
 ****/
 
+bool Alarm::operator==(const Alarm& that) const
+{
+  return (text==that.text)
+      && (audio_url==that.audio_url)
+          && (this->time==that.time);
+}
+
+bool Alarm::has_sound() const
+{
+    return !audio_url.empty();
+}
+
+bool Alarm::has_text() const
+{
+    return !text.empty();
+}
+
 bool Appointment::operator==(const Appointment& that) const
 {
-    return (color==that.color)
-        && (summary==that.summary)
-        && (url==that.url)
+    return (type==that.type)
         && (uid==that.uid)
-        && (has_alarms==that.has_alarms)
+        && (color==that.color)
+        && (summary==that.summary)
         && (begin==that.begin)
-        && (end==that.end);
+        && (end==that.end)
+        && (alarms==that.alarms);
 }
 
 /****
