@@ -25,6 +25,8 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
+#include <algorithm>
+#include <iterator>
 #include <vector>
 
 namespace unity {
@@ -56,12 +58,90 @@ GMenuModel* Menu::menu_model()
     return G_MENU_MODEL(m_menu);
 }
 
+/**
+ * To avoid a giant menu on the PC, and to avoid pushing lower menu items
+ * off-screen on the phone, the menu should the
+ * next five calendar events, if any.
+ *
+ * The list might include multiple occurrences of the same event (bug 1515821).
+ */
+std::vector<Appointment>
+Menu::get_display_appointments(const std::vector<Appointment>& appointments_in,
+                               const DateTime& now,
+                               unsigned int max_items)
+{
+    std::vector<Appointment> appointments;
+    std::copy_if(appointments_in.begin(),
+                 appointments_in.end(),
+                 std::back_inserter(appointments),
+                 [now](const Appointment& a){return a.end >= now;});
+
+    if (appointments.size() > max_items)
+    {
+        const auto next_minute = now.add_full(0,0,0,0,1,-now.seconds());
+        const auto start_of_day = now.start_of_day();
+        const auto end_of_day = now.end_of_day();
+
+        /*
+         * If there are more than five, the events shown should be, in order of priority:
+         * 1. any events that start or end (bug 1329048) after the current minute today;
+         * 2. any full-day events that span all of today (bug 1302004);
+         * 3. any events that start or end tomorrow;
+         * 4. any events that start or end the day after tomorrow; and so on.
+         */
+        auto compare = [next_minute, start_of_day, end_of_day](
+            const Appointment& a,
+            const Appointment& b)
+        {
+            const bool a_later_today = (a.begin >= next_minute) || (a.end <= end_of_day);
+            const bool b_later_today = (b.begin >= next_minute) || (b.end <= end_of_day);
+            if (a_later_today != b_later_today)
+                return a_later_today;
+
+            const bool a_full_day_today = (a.begin <= start_of_day) && (end_of_day <= a.end);
+            const bool b_full_day_today = (b.begin <= start_of_day) && (end_of_day <= b.end);
+            if (a_full_day_today != b_full_day_today)
+                return a_full_day_today;
+
+            const bool a_after_today = (a.begin > end_of_day) || (a.end > end_of_day);
+            const bool b_after_today = (a.begin > end_of_day) || (a.end > end_of_day);
+            if (a_after_today != b_after_today)
+                return a_after_today;
+            if (a.begin != b.begin)
+                return a.begin < b.begin;
+            if (b.end != b.end)
+                return a.end < b.end;
+
+            return false;
+        };
+        std::sort(appointments.begin(), appointments.end(), compare);
+        appointments.resize(max_items);
+    }
+
+    /*
+     * However, the display order should be the reverse: full-day events
+     * first (since they start first), part-day events afterward in
+     * chronological order. If multiple events have exactly the same start+end
+     * time, they should be sorted alphabetically.
+     */
+    auto compare = [](const Appointment& a, const Appointment& b)
+    {
+        if (a.begin != b.begin)
+            return a.begin < b.begin;
+
+        if (a.end != b.end)
+            return a.end < b.end;
+
+        return a.summary < b.summary;
+    };
+    std::sort(appointments.begin(), appointments.end(), compare);
+    return appointments;
+}
 
 /****
 *****
 ****/
 
-
 #define ALARM_ICON_NAME "alarm-clock"
 #define CALENDAR_ICON_NAME "calendar"
 
@@ -141,26 +221,20 @@ protected:
 
     void update_upcoming()
     {
-        // The usual case is on desktop (and /only/ case on phone)
-        // is that we're looking at the current date and want to see
-        // "the next five calendar events, if any."
-        //
-        // However on the Desktop when the user clicks onto a different
-        // calendar date, show the next five calendar events starting
-        // from the beginning of that clicked day.
-        DateTime begin;
+        // The usual case is to show events germane to the current time.
+        // However when the user clicks onto a different calendar date,
+        // we pick events starting from the beginning of that clicked day.
         const auto now = m_state->clock->localtime();
         const auto calendar_day = m_state->calendar_month->month().get();
-        if ((profile() == Desktop) && !DateTime::is_same_day(now, calendar_day))
-            begin = calendar_day.add_full (0, 0, 0, -calendar_day.hour(), -calendar_day.minute(), -calendar_day.seconds());
-        else
-            begin = now.add_full (0, 0, 0, 0, 0, -now.seconds());
+        const auto begin = DateTime::is_same_day(now, calendar_day)
+            ? now.start_of_minute()
+            : calendar_day.start_of_day();
+
+        auto upcoming = get_display_appointments(
+            m_state->calendar_upcoming->appointments().get(),
+            begin
+        );
 
-        std::vector<Appointment> upcoming;
-        for(const auto& a : m_state->calendar_upcoming->appointments().get())
-            if (begin <= a.begin)
-                upcoming.push_back(a);
- 
         if (m_upcoming != upcoming)
         {
             m_upcoming.swap(upcoming);
@@ -235,7 +309,7 @@ private:
     GMenuModel* create_calendar_section(Profile profile)
     {
         const bool show_calendar = m_state->settings->show_calendar.get() &&
-                                   ((profile == Desktop) || (profile == DesktopGreeter));
+                                   ((profile == Desktop) || (profile == DesktopGreeter) || (profile == Phone));
         auto menu = g_menu_new();
 
         const char * action_name;
@@ -316,7 +390,7 @@ private:
             g_menu_item_set_attribute (menu_item, "x-canonical-time", "x", unix_time);
             g_menu_item_set_attribute (menu_item, "x-canonical-time-format", "s", fmt.c_str());
 
-            if (appt.has_alarms)
+            if (appt.is_ubuntu_alarm())
             {
                 g_menu_item_set_attribute (menu_item, "x-canonical-type", "s", "com.canonical.indicator.alarm");
                 g_menu_item_set_attribute_value(menu_item, G_MENU_ATTRIBUTE_ICON, get_serialized_alarm_icon());
@@ -329,9 +403,12 @@ private:
             if (!appt.color.empty())
                 g_menu_item_set_attribute (menu_item, "x-canonical-color", "s", appt.color.c_str());
 
-            if (action_name != nullptr)
+            if (action_name != nullptr) {
                 g_menu_item_set_action_and_target_value (menu_item, action_name,
-                                                         g_variant_new_string (appt.uid.c_str()));
+                                                         g_variant_new ("(sx)",
+                                                                        appt.uid.c_str(),
+                                                                        unix_time));
+            }
 
             g_menu_append_item (menu, menu_item);
             g_object_unref (menu_item);
@@ -374,7 +451,7 @@ private:
     {
         GMenu* menu = g_menu_new();
 
-        if (profile == Desktop)
+        if (profile == Desktop || profile == Phone)
         {
             const auto now = m_state->clock->localtime();
 
@@ -411,7 +488,7 @@ private:
             action_name = nullptr;
 
         if (action_name != nullptr)
-            g_menu_append (menu, _("Time & Date settings…"), action_name);
+            g_menu_append (menu, _("Time & Date Settings…"), action_name);
 
         return G_MENU_MODEL (menu);
     }
@@ -509,16 +586,16 @@ protected:
     GVariant* create_header_state()
     {
         // are there alarms?
-        bool has_alarms = false;
+        bool has_ubuntu_alarms = false;
         for(const auto& appointment : m_upcoming)
-            if((has_alarms = appointment.has_alarms))
+            if((has_ubuntu_alarms = appointment.is_ubuntu_alarm()))
                 break;
 
         GVariantBuilder b;
         g_variant_builder_init(&b, G_VARIANT_TYPE_VARDICT);
-        g_variant_builder_add(&b, "{sv}", "title", g_variant_new_string (_("Upcoming")));
+        g_variant_builder_add(&b, "{sv}", "title", g_variant_new_string (_("Time & Date")));
         g_variant_builder_add(&b, "{sv}", "visible", g_variant_new_boolean (TRUE));
-        if (has_alarms)
+        if (has_ubuntu_alarms)
         {
             auto label = m_formatter->header.get();
             auto a11y = g_strdup_printf(_("%s (has alarms)"), label.c_str());
@@ -590,7 +667,7 @@ MenuFactory::buildMenu(Menu::Profile profile)
         g_warn_if_reached();
         break;
     }
-    
+
     return menu;
 }
 
