@@ -39,7 +39,7 @@ protected:
     std::shared_ptr<MenuFactory> m_menu_factory;
     std::vector<std::shared_ptr<Menu>> m_menus;
 
-    virtual void SetUp()
+    void SetUp() override
     {
         super::SetUp();
 
@@ -49,7 +49,7 @@ protected:
             m_menus.push_back(m_menu_factory->buildMenu(Menu::Profile(i)));
     }
 
-    virtual void TearDown()
+    void TearDown() override
     {
         m_menus.clear();
         m_menu_factory.reset();
@@ -102,7 +102,7 @@ protected:
         else
             expected_action = nullptr;
 
-        const auto calendar_expected = ((profile == Menu::Desktop) || (profile == Menu::DesktopGreeter))
+        const auto calendar_expected = ((profile == Menu::Desktop) || (profile == Menu::DesktopGreeter) || (profile == Menu::Phone))
                                     && (m_state->settings->show_calendar.get());
 
         // get the calendar section
@@ -151,9 +151,7 @@ protected:
 
         // now change the clock and see if the date label changes appropriately
 
-        auto gdt_tomorrow = g_date_time_add_days(now.get(), 1);
-        auto tomorrow = DateTime(gdt_tomorrow);
-        g_date_time_unref(gdt_tomorrow);
+        auto tomorrow = now.add_days(1).start_of_day();
         m_mock_state->mock_clock->set_localtime(tomorrow);
         wait_msec();
 
@@ -182,16 +180,14 @@ private:
     std::vector<Appointment> build_some_appointments()
     {
         const auto now = m_state->clock->localtime();
-        auto gdt_tomorrow = g_date_time_add_days(now.get(), 1);
-        const auto tomorrow = DateTime(gdt_tomorrow);
-        g_date_time_unref(gdt_tomorrow);
+        const auto tomorrow = now.add_days(1);
 
         Appointment a1; // an alarm clock appointment
         a1.color = "red";
         a1.summary = "Alarm";
         a1.summary = "http://www.example.com/";
         a1.uid = "example";
-        a1.has_alarms = true;
+        a1.type = Appointment::UBUNTU_ALARM;
         a1.begin = a1.end = tomorrow;
 
         Appointment a2; // a non-alarm appointment
@@ -199,7 +195,7 @@ private:
         a2.summary = "Other Text";
         a2.summary = "http://www.monkey.com/";
         a2.uid = "monkey";
-        a2.has_alarms = false;
+        a1.type = Appointment::EVENT;
         a2.begin = a2.end = tomorrow;
 
         return std::vector<Appointment>({a1, a2});
@@ -212,7 +208,7 @@ private:
         //  confirm it has the right x-canonical-type
         gchar * str = nullptr;
         g_menu_model_get_item_attribute(section, index, "x-canonical-type", "s", &str);
-        if (appt.has_alarms)
+        if (appt.is_ubuntu_alarm())
             EXPECT_STREQ("com.canonical.indicator.alarm", str);
         else
             EXPECT_STREQ("com.canonical.indicator.appointment", str);
@@ -245,7 +241,7 @@ private:
         g_clear_pointer(&str, g_free);
 
         // confirm that alarms have an icon
-        if (appt.has_alarms)
+        if (appt.is_ubuntu_alarm())
         {
             auto v = g_menu_model_get_item_attribute_value(section,
                                                            index,
@@ -424,7 +420,7 @@ protected:
 
     void InspectLocations(GMenuModel* menu_model, Menu::Profile profile)
     {
-        const bool locations_expected = profile == Menu::Desktop;
+        const bool locations_expected = (profile == Menu::Desktop) || (profile == Menu::Phone);
 
         // when there aren't any locations, confirm the menu is empty
         const std::vector<Location> empty;
@@ -484,7 +480,7 @@ TEST_F(MenuFixture, HelloWorld)
     EXPECT_EQ(Menu::NUM_PROFILES, m_menus.size());
     for (int i=0; i<Menu::NUM_PROFILES; i++)
     {
-        EXPECT_TRUE(m_menus[i] != false);
+        EXPECT_TRUE(m_menus[i].get() != nullptr);
         EXPECT_TRUE(m_menus[i]->menu_model() != nullptr);
         EXPECT_EQ(i, m_menus[i]->profile());
     }
